@@ -25,8 +25,9 @@
 #include "gck-ssh-private-key.h"
 
 #include "gck/gck-attributes.h"
-#include "gck/gck-authenticator.h"
+#include "gck/gck-credential.h"
 #include "gck/gck-manager.h"
+#include "gck/gck-module.h"
 #include "gck/gck-object.h"
 #include "gck/gck-sexp.h"
 #include "gck/gck-util.h"
@@ -42,8 +43,8 @@ enum {
 };
 
 struct _GckSshPrivateKey {
-	GckPrivateKey parent;
-	
+	GckPrivateXsaKey parent;
+
 	GckSshPublicKey *pubkey;
 	gchar *label;
 	guchar *private_data;
@@ -52,7 +53,7 @@ struct _GckSshPrivateKey {
 	gboolean is_encrypted;
 };
 
-G_DEFINE_TYPE (GckSshPrivateKey, gck_ssh_private_key, GCK_TYPE_PRIVATE_KEY);
+G_DEFINE_TYPE (GckSshPrivateKey, gck_ssh_private_key, GCK_TYPE_PRIVATE_XSA_KEY);
 
 /* -----------------------------------------------------------------------------
  * INTERNAL 
@@ -107,8 +108,8 @@ realize_and_take_data (GckSshPrivateKey *self, gcry_sexp_t sexp, gchar *comment,
 
 	/* The base public key gets setup. */
 	wrapper = gck_sexp_new (sexp);
-	gck_key_set_base_sexp (GCK_KEY (self), wrapper);
-	gck_key_set_base_sexp (GCK_KEY (self->pubkey), wrapper);
+	gck_sexp_key_set_base (GCK_SEXP_KEY (self), wrapper);
+	gck_sexp_key_set_base (GCK_SEXP_KEY (self->pubkey), wrapper);
 	gck_sexp_unref (wrapper);
 
 	/* Own the comment */
@@ -125,7 +126,7 @@ realize_and_take_data (GckSshPrivateKey *self, gcry_sexp_t sexp, gchar *comment,
 	self->is_encrypted = TRUE;
 	if (unlock_private_key (self, "", 0, &wrapper) == CKR_OK) {
 		self->is_encrypted = FALSE;
-		gck_private_key_set_unlocked_private (GCK_PRIVATE_KEY (self), wrapper);
+		gck_private_xsa_key_set_unlocked_private (GCK_PRIVATE_XSA_KEY (self), wrapper);
 		gck_sexp_unref (wrapper);
 	}
 }
@@ -159,7 +160,7 @@ gck_ssh_private_key_get_attribute (GckObject *base, GckSession *session, CK_ATTR
 }
 
 static CK_RV
-gck_ssh_private_key_unlock (GckObject *base, GckAuthenticator *auth)
+gck_ssh_private_key_unlock (GckObject *base, GckCredential *cred)
 {
 	GckSshPrivateKey *self = GCK_SSH_PRIVATE_KEY (base);
 	const gchar *password;
@@ -170,17 +171,24 @@ gck_ssh_private_key_unlock (GckObject *base, GckAuthenticator *auth)
 	if (!self->is_encrypted)
 		return CKR_OK;
 
-	password = gck_authenticator_get_password (auth, &n_password);
+	password = gck_credential_get_password (cred, &n_password);
 	rv = unlock_private_key (self, password, n_password, &wrapper);
 
 	if (rv == CKR_OK) {
-		gck_private_key_set_locked_private (GCK_PRIVATE_KEY (self), auth, wrapper);
+		gck_private_xsa_key_set_locked_private (GCK_PRIVATE_XSA_KEY (self), cred, wrapper);
 		gck_sexp_unref (wrapper);
 	}
 
 	return rv;
 }
 
+static void
+gck_ssh_private_key_expose (GckObject *base, gboolean expose)
+{
+	GCK_OBJECT_CLASS (gck_ssh_private_key_parent_class)->expose_object (base, expose);
+	gck_object_expose (GCK_OBJECT (GCK_SSH_PRIVATE_KEY (base)->pubkey), expose);
+}
+
 static GObject* 
 gck_ssh_private_key_constructor (GType type, guint n_props, GObjectConstructParam *props) 
 {
@@ -281,6 +289,7 @@ gck_ssh_private_key_class_init (GckSshPrivateKeyClass *klass)
 	
 	gck_class->get_attribute = gck_ssh_private_key_get_attribute;
 	gck_class->unlock = gck_ssh_private_key_unlock;
+	gck_class->expose_object = gck_ssh_private_key_expose;
 	
 	g_object_class_install_property (gobject_class, PROP_LABEL,
 	           g_param_spec_string ("label", "Label", "Object Label", 
@@ -299,7 +308,7 @@ GckSshPrivateKey*
 gck_ssh_private_key_new (GckModule *module, const gchar *unique)
 {
 	return g_object_new (GCK_TYPE_SSH_PRIVATE_KEY, "unique", unique, 
-	                     "module", module, NULL);
+	                     "module", module, "manager", gck_module_get_manager (module), NULL);
 }
 
 gboolean
